window.Gleam.initSearch([{"doc":"emel","title":"emel","content":"# emel\n\nTurn data into functions! A simple and functional **machine learning** library, (re)implemented in **Gleam** for the **Erlang** ecosystem.\n\n## Installation\n\n* For **Erlang** projects, add the dependency into `rebar.config`:\n\n```erlang\n{deps, [\n    {emel, &quot;1.0.1&quot;}\n]}.\n```\n\n* For **Elixir** projects, add the dependency into `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:emel, &quot;~&gt; 1.0.1&quot;}\n  ]\nend\n```\n\n* For **Gleam** projects, run `gleam add emel`.\n\n## Examples\n\n### Erlang\n\n```erlang\nData = [\n  {[7.0, 7.0], &quot;bad&quot;},\n  {[7.0, 4.0], &quot;bad&quot;},\n  {[3.0, 4.0], &quot;good&quot;},\n  {[1.0, 4.0], &quot;good&quot;}\n],\nF = emel@ml@k_nearest_neighbors:classifier(Data, 3),\nF([3.0, 7.0]). % &quot;good&quot;\n```\n \n### Elixir\n\n```elixir\ndata = [\n  {[1.794638, 15.15426], 0.510998918},\n  {[3.220726, 229.6516], 105.6583692},\n  {[5.780040, 3480.201], 1776.99}\n]\nf = :emel@ml@linear_regression.predictor(data)\nf.([3.0, 230.0]) # 106.74114058686602\n```\n\n### Gleam\n\n```gleam\nimport emel/ml/neural_network as nn\nlet data = [\n  #([0.8, 0.0, 0.0], &quot;x&quot;),\n  #([0.0, 0.9, 0.0], &quot;y&quot;),\n  #([0.0, 0.0, 0.8], &quot;z&quot;),\n]\nlet f = nn.classifier(\n    data,\n    [4],           // hidden layers\n    0.01,          // learning rate\n    0.1,           // error threshold\n    1000           // maximum iterations\n)\nf([0.0, 0.8, 0.0]) // &quot;y&quot;\n```\n\n## Documentation\n\nThe [documentation](https://hexdocs.pm/emel/1.0.0/) describes all the public functions. **Gleam** developers can pay attention to the type annotations. **Erlang** and **Elixir** devs may take a look at the examples which are written in **Erlang**.\n\n### Implemented Algorithms\n\n * [Linear Regression](https://hexdocs.pm/emel/1.0.0/emel/ml/linear_regression.html)\n * [K Nearest Neighbors](https://hexdocs.pm/emel/1.0.0/emel/ml/k_nearest_neighbors.html)\n * [Decision Tree](https://hexdocs.pm/emel/1.0.0/emel/ml/decision_tree.html)\n * [Naive Bayes](https://hexdocs.pm/emel/1.0.0/emel/ml/naive_bayes.html)\n * [K Means](https://hexdocs.pm/emel/1.0.0/emel/ml/k_means.html)\n * [Perceptron](https://hexdocs.pm/emel/1.0.0/emel/ml/perceptron.html)\n * [Logistic Regression](https://hexdocs.pm/emel/1.0.0/emel/ml/logistic_regression.html)\n * [Neural Network](https://hexdocs.pm/emel/1.0.0/emel/ml/neural_network.html)\n\n### Mathematics\n\n* [Algebra](https://hexdocs.pm/emel/1.0.0/emel/math/algebra.html)\n* [Geometry](https://hexdocs.pm/emel/1.0.0/emel/math/geometry.html)\n* [Statistics](https://hexdocs.pm/emel/1.0.0/emel/math/statistics.html)\n\n_For the documentation of the previous version (`0.3.0`), click [here](https://hexdocs.pm/emel/0.3.0/)._\n","url":"index.html"},{"doc":"emel/utils/result","title":"unsafe_f","content":"pub fn unsafe_f(f: fn(a) -&gt; Result(b, c)) -&gt; fn(a) -&gt; b\n\nSynonyms:\nemel/utils/result.unsafe_f\nemel/utils/result unsafe_f","url":"emel/utils/result.html#unsafe_f"},{"doc":"emel/utils/result","title":"unsafe_get","content":"pub fn unsafe_get(m: Dict(a, b), k: a) -&gt; b\n\nSynonyms:\nemel/utils/result.unsafe_get\nemel/utils/result unsafe_get","url":"emel/utils/result.html#unsafe_get"},{"doc":"emel/utils/result","title":"unsafe_res","content":"pub fn unsafe_res(res: Result(a, b)) -&gt; a\n\nSynonyms:\nemel/utils/result.unsafe_res\nemel/utils/result unsafe_res","url":"emel/utils/result.html#unsafe_res"},{"doc":"emel/utils/result","title":"emel/utils/result","content":"","url":"emel/utils/result.html"},{"doc":"emel/utils/zlist","title":"avg","content":"pub fn avg(zl: ZList(Float)) -&gt; Result(Float, Nil)\n\nSynonyms:\nemel/utils/zlist.avg\nemel/utils/zlist avg","url":"emel/utils/zlist.html#avg"},{"doc":"emel/utils/zlist","title":"delete_at","content":"pub fn delete_at(zl: ZList(a), i: Int) -&gt; ZList(a)\n\nSynonyms:\nemel/utils/zlist.delete_at\nemel/utils/zlist delete_at","url":"emel/utils/zlist.html#delete_at"},{"doc":"emel/utils/zlist","title":"freqs_with_size","content":"pub fn freqs_with_size(zl: ZList(a)) -&gt; #(ZList(#(a, Int)), Int)\n\nSynonyms:\nemel/utils/zlist.freqs_with_size\nemel/utils/zlist freqs_with_size","url":"emel/utils/zlist.html#freqs_with_size"},{"doc":"emel/utils/zlist","title":"group_by","content":"pub fn group_by(\n  zl: ZList(a),\n  f: fn(a) -&gt; b,\n) -&gt; ZList(#(b, ZList(a)))\n\nSynonyms:\nemel/utils/zlist.group_by\nemel/utils/zlist group_by","url":"emel/utils/zlist.html#group_by"},{"doc":"emel/utils/zlist","title":"max_by","content":"pub fn max_by(zl: ZList(a), f: fn(a) -&gt; Float) -&gt; Result(a, Nil)\n\nSynonyms:\nemel/utils/zlist.max_by\nemel/utils/zlist max_by","url":"emel/utils/zlist.html#max_by"},{"doc":"emel/utils/zlist","title":"min_by","content":"pub fn min_by(zl: ZList(a), f: fn(a) -&gt; Float) -&gt; Result(a, Nil)\n\nSynonyms:\nemel/utils/zlist.min_by\nemel/utils/zlist min_by","url":"emel/utils/zlist.html#min_by"},{"doc":"emel/utils/zlist","title":"replace_at","content":"pub fn replace_at(zl: ZList(a), i: Int, elem: a) -&gt; ZList(a)\n\nSynonyms:\nemel/utils/zlist.replace_at\nemel/utils/zlist replace_at","url":"emel/utils/zlist.html#replace_at"},{"doc":"emel/utils/zlist","title":"shuffle","content":"pub fn shuffle(zl: ZList(a), seed: Int) -&gt; ZList(a)\n\nSynonyms:\nemel/utils/zlist.shuffle\nemel/utils/zlist shuffle","url":"emel/utils/zlist.html#shuffle"},{"doc":"emel/utils/zlist","title":"to_list_of_lists","content":"pub fn to_list_of_lists(zl: ZList(ZList(a))) -&gt; List(List(a))\n\nSynonyms:\nemel/utils/zlist.to_list_of_lists\nemel/utils/zlist to_list_of_lists","url":"emel/utils/zlist.html#to_list_of_lists"},{"doc":"emel/utils/zlist","title":"to_zlist_of_zlists","content":"pub fn to_zlist_of_zlists(ls: List(List(a))) -&gt; ZList(ZList(a))\n\nSynonyms:\nemel/utils/zlist.to_zlist_of_zlists\nemel/utils/zlist to_zlist_of_zlists","url":"emel/utils/zlist.html#to_zlist_of_zlists"},{"doc":"emel/utils/zlist","title":"uniq","content":"pub fn uniq(zl: ZList(a)) -&gt; ZList(a)\n\nSynonyms:\nemel/utils/zlist.uniq\nemel/utils/zlist uniq","url":"emel/utils/zlist.html#uniq"},{"doc":"emel/utils/zlist","title":"emel/utils/zlist","content":"","url":"emel/utils/zlist.html"},{"doc":"emel/lazy/math/algebra","title":"cramer_solution","content":"pub fn cramer_solution(\n  coefficients: ZList(ZList(Float)),\n  constants: ZList(Float),\n) -&gt; Result(ZList(Float), String)\n\nSynonyms:\nemel/lazy/math/algebra.cramer_solution\nemel/lazy/math/algebra cramer_solution","url":"emel/lazy/math/algebra.html#cramer_solution"},{"doc":"emel/lazy/math/algebra","title":"determinant","content":"pub fn determinant(matrix: ZList(ZList(Float))) -&gt; Float\n\nSynonyms:\nemel/lazy/math/algebra.determinant\nemel/lazy/math/algebra determinant","url":"emel/lazy/math/algebra.html#determinant"},{"doc":"emel/lazy/math/algebra","title":"first_minor","content":"pub fn first_minor(\n  matrix: ZList(ZList(Float)),\n  i: Int,\n  j: Int,\n) -&gt; ZList(ZList(Float))\n\nSynonyms:\nemel/lazy/math/algebra.first_minor\nemel/lazy/math/algebra first_minor","url":"emel/lazy/math/algebra.html#first_minor"},{"doc":"emel/lazy/math/algebra","title":"transpose","content":"pub fn transpose(matrix: ZList(ZList(a))) -&gt; ZList(ZList(a))\n\nSynonyms:\nemel/lazy/math/algebra.transpose\nemel/lazy/math/algebra transpose","url":"emel/lazy/math/algebra.html#transpose"},{"doc":"emel/lazy/math/algebra","title":"emel/lazy/math/algebra","content":"","url":"emel/lazy/math/algebra.html"},{"doc":"emel/lazy/math/geometry","title":"centroid","content":"pub fn centroid(points: ZList(ZList(Float))) -&gt; ZList(Float)\n\nSynonyms:\nemel/lazy/math/geometry.centroid\nemel/lazy/math/geometry centroid","url":"emel/lazy/math/geometry.html#centroid"},{"doc":"emel/lazy/math/geometry","title":"dot_product","content":"pub fn dot_product(x: ZList(Float), y: ZList(Float)) -&gt; Float\n\nSynonyms:\nemel/lazy/math/geometry.dot_product\nemel/lazy/math/geometry dot_product","url":"emel/lazy/math/geometry.html#dot_product"},{"doc":"emel/lazy/math/geometry","title":"euclidean_distance","content":"pub fn euclidean_distance(\n  x: ZList(Float),\n  y: ZList(Float),\n) -&gt; Float\n\nSynonyms:\nemel/lazy/math/geometry.euclidean_distance\nemel/lazy/math/geometry euclidean_distance","url":"emel/lazy/math/geometry.html#euclidean_distance"},{"doc":"emel/lazy/math/geometry","title":"magnitude","content":"pub fn magnitude(vector: ZList(Float)) -&gt; Float\n\nSynonyms:\nemel/lazy/math/geometry.magnitude\nemel/lazy/math/geometry magnitude","url":"emel/lazy/math/geometry.html#magnitude"},{"doc":"emel/lazy/math/geometry","title":"nearest_neighbor","content":"pub fn nearest_neighbor(\n  point: ZList(Float),\n  neighbors: ZList(ZList(Float)),\n) -&gt; ZList(Float)\n\nSynonyms:\nemel/lazy/math/geometry.nearest_neighbor\nemel/lazy/math/geometry nearest_neighbor","url":"emel/lazy/math/geometry.html#nearest_neighbor"},{"doc":"emel/lazy/math/geometry","title":"emel/lazy/math/geometry","content":"","url":"emel/lazy/math/geometry.html"},{"doc":"emel/lazy/math/statistics","title":"classical_probability","content":"pub fn classical_probability(\n  observations: ZList(a),\n  event: a,\n) -&gt; Float\n\nSynonyms:\nemel/lazy/math/statistics.classical_probability\nemel/lazy/math/statistics classical_probability","url":"emel/lazy/math/statistics.html#classical_probability"},{"doc":"emel/lazy/math/statistics","title":"entropy","content":"pub fn entropy(probability_values: ZList(Float)) -&gt; Float\n\nSynonyms:\nemel/lazy/math/statistics.entropy\nemel/lazy/math/statistics entropy","url":"emel/lazy/math/statistics.html#entropy"},{"doc":"emel/lazy/math/statistics","title":"log","content":"pub fn log(x: Float) -&gt; Float\n\nSynonyms:\nemel/lazy/math/statistics.log\nemel/lazy/math/statistics log","url":"emel/lazy/math/statistics.html#log"},{"doc":"emel/lazy/math/statistics","title":"mean_absolute_error","content":"pub fn mean_absolute_error(\n  predictions: ZList(Float),\n  observations: ZList(Float),\n) -&gt; Float\n\nSynonyms:\nemel/lazy/math/statistics.mean_absolute_error\nemel/lazy/math/statistics mean_absolute_error","url":"emel/lazy/math/statistics.html#mean_absolute_error"},{"doc":"emel/lazy/math/statistics","title":"emel/lazy/math/statistics","content":"","url":"emel/lazy/math/statistics.html"},{"doc":"emel/lazy/ml/decision_tree","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(Dict(String, String)),\n  discrete_attributes: ZList(String),\n  class: String,\n) -&gt; fn(Dict(String, String)) -&gt; String\n\nSynonyms:\nemel/lazy/ml/decision_tree.classifier\nemel/lazy/ml/decision_tree classifier","url":"emel/lazy/ml/decision_tree.html#classifier"},{"doc":"emel/lazy/ml/decision_tree","title":"decision_tree","content":"pub fn decision_tree(\n  dataset: ZList(Dict(String, String)),\n  attributes: ZList(String),\n  class: String,\n) -&gt; ZList(Dict(String, String))\n\nSynonyms:\nemel/lazy/ml/decision_tree.decision_tree\nemel/lazy/ml/decision_tree decision_tree","url":"emel/lazy/ml/decision_tree.html#decision_tree"},{"doc":"emel/lazy/ml/decision_tree","title":"entropy_with_size","content":"pub fn entropy_with_size(\n  dataset: ZList(Dict(String, String)),\n  class_attr: String,\n) -&gt; #(Float, Int)\n\nSynonyms:\nemel/lazy/ml/decision_tree.entropy_with_size\nemel/lazy/ml/decision_tree entropy_with_size","url":"emel/lazy/ml/decision_tree.html#entropy_with_size"},{"doc":"emel/lazy/ml/decision_tree","title":"feature_entropy","content":"pub fn feature_entropy(\n  dataset: ZList(Dict(String, String)),\n  class_attr: String,\n  feature: String,\n  dataset_size: Int,\n) -&gt; Float\n\nSynonyms:\nemel/lazy/ml/decision_tree.feature_entropy\nemel/lazy/ml/decision_tree feature_entropy","url":"emel/lazy/ml/decision_tree.html#feature_entropy"},{"doc":"emel/lazy/ml/decision_tree","title":"emel/lazy/ml/decision_tree","content":"","url":"emel/lazy/ml/decision_tree.html"},{"doc":"emel/lazy/ml/k_means","title":"centroids_with_clusters","content":"pub fn centroids_with_clusters(\n  points: ZList(ZList(Float)),\n  k: Int,\n  seed: Int,\n) -&gt; ZList(#(ZList(Float), ZList(ZList(Float))))\n\nSynonyms:\nemel/lazy/ml/k_means.centroids_with_clusters\nemel/lazy/ml/k_means centroids_with_clusters","url":"emel/lazy/ml/k_means.html#centroids_with_clusters"},{"doc":"emel/lazy/ml/k_means","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(ZList(Float)),\n  k: Int,\n  seed: Int,\n) -&gt; fn(ZList(Float)) -&gt; Int\n\nSynonyms:\nemel/lazy/ml/k_means.classifier\nemel/lazy/ml/k_means classifier","url":"emel/lazy/ml/k_means.html#classifier"},{"doc":"emel/lazy/ml/k_means","title":"clusters","content":"pub fn clusters(\n  points: ZList(ZList(Float)),\n  k: Int,\n  seed: Int,\n) -&gt; ZList(ZList(ZList(Float)))\n\nSynonyms:\nemel/lazy/ml/k_means.clusters\nemel/lazy/ml/k_means clusters","url":"emel/lazy/ml/k_means.html#clusters"},{"doc":"emel/lazy/ml/k_means","title":"emel/lazy/ml/k_means","content":"","url":"emel/lazy/ml/k_means.html"},{"doc":"emel/utils/min_sorted_zlist","title":"extract","content":"pub fn extract(mszl: ZList(Option(#(a, Float)))) -&gt; ZList(a)\n\nSynonyms:\nemel/utils/min_sorted_zlist.extract\nemel/utils/min_sorted_zlist extract","url":"emel/utils/min_sorted_zlist.html#extract"},{"doc":"emel/utils/min_sorted_zlist","title":"new","content":"pub fn new(size: Int) -&gt; ZList(Option(#(a, Float)))\n\nSynonyms:\nemel/utils/min_sorted_zlist.new\nemel/utils/min_sorted_zlist new","url":"emel/utils/min_sorted_zlist.html#new"},{"doc":"emel/utils/min_sorted_zlist","title":"updated","content":"pub fn updated(\n  mszl: ZList(Option(#(a, Float))),\n  el_with_weight: #(a, Float),\n) -&gt; ZList(Option(#(a, Float)))\n\nSynonyms:\nemel/utils/min_sorted_zlist.updated\nemel/utils/min_sorted_zlist updated","url":"emel/utils/min_sorted_zlist.html#updated"},{"doc":"emel/utils/min_sorted_zlist","title":"emel/utils/min_sorted_zlist","content":"","url":"emel/utils/min_sorted_zlist.html"},{"doc":"emel/lazy/ml/k_nearest_neighbors","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(#(ZList(Float), String)),\n  k: Int,\n) -&gt; fn(ZList(Float)) -&gt; String\n\nSynonyms:\nemel/lazy/ml/k_nearest_neighbors.classifier\nemel/lazy/ml/k_nearest_neighbors classifier","url":"emel/lazy/ml/k_nearest_neighbors.html#classifier"},{"doc":"emel/lazy/ml/k_nearest_neighbors","title":"k_nearest_neighbors","content":"pub fn k_nearest_neighbors(\n  dataset: ZList(#(ZList(Float), a)),\n  item: ZList(Float),\n  k: Int,\n) -&gt; ZList(#(ZList(Float), a))\n\nSynonyms:\nemel/lazy/ml/k_nearest_neighbors.k_nearest_neighbors\nemel/lazy/ml/k_nearest_neighbors k_nearest_neighbors","url":"emel/lazy/ml/k_nearest_neighbors.html#k_nearest_neighbors"},{"doc":"emel/lazy/ml/k_nearest_neighbors","title":"predictor","content":"pub fn predictor(\n  dataset: ZList(#(ZList(Float), Float)),\n  k: Int,\n) -&gt; fn(ZList(Float)) -&gt; Float\n\nSynonyms:\nemel/lazy/ml/k_nearest_neighbors.predictor\nemel/lazy/ml/k_nearest_neighbors predictor","url":"emel/lazy/ml/k_nearest_neighbors.html#predictor"},{"doc":"emel/lazy/ml/k_nearest_neighbors","title":"emel/lazy/ml/k_nearest_neighbors","content":"","url":"emel/lazy/ml/k_nearest_neighbors.html"},{"doc":"emel/lazy/ml/linear_regression","title":"equation_terms","content":"pub fn equation_terms(\n  points: ZList(ZList(Float)),\n) -&gt; ZList(ZList(Float))\n\nSynonyms:\nemel/lazy/ml/linear_regression.equation_terms\nemel/lazy/ml/linear_regression equation_terms","url":"emel/lazy/ml/linear_regression.html#equation_terms"},{"doc":"emel/lazy/ml/linear_regression","title":"predictor","content":"pub fn predictor(\n  dataset: ZList(#(ZList(Float), Float)),\n) -&gt; fn(ZList(Float)) -&gt; Float\n\nSynonyms:\nemel/lazy/ml/linear_regression.predictor\nemel/lazy/ml/linear_regression predictor","url":"emel/lazy/ml/linear_regression.html#predictor"},{"doc":"emel/lazy/ml/linear_regression","title":"regression_coefficients","content":"pub fn regression_coefficients(\n  points: ZList(ZList(Float)),\n) -&gt; Result(ZList(Float), String)\n\nSynonyms:\nemel/lazy/ml/linear_regression.regression_coefficients\nemel/lazy/ml/linear_regression regression_coefficients","url":"emel/lazy/ml/linear_regression.html#regression_coefficients"},{"doc":"emel/lazy/ml/linear_regression","title":"emel/lazy/ml/linear_regression","content":"","url":"emel/lazy/ml/linear_regression.html"},{"doc":"emel/lazy/ml/neural_network","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(#(ZList(Float), String)),\n  hidden_layers: ZList(Int),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n  seed: Int,\n) -&gt; fn(ZList(Float)) -&gt; String\n\nSynonyms:\nemel/lazy/ml/neural_network.classifier\nemel/lazy/ml/neural_network classifier","url":"emel/lazy/ml/neural_network.html#classifier"},{"doc":"emel/lazy/ml/neural_network","title":"iterate","content":"pub fn iterate(\n  net: ZList(ZList(Neuron)),\n  learning_rate: Float,\n  inputs: ZList(ZList(Float)),\n  expected_outputs: ZList(ZList(Float)),\n  error_threshold: Float,\n  countdown: Int,\n) -&gt; ZList(ZList(Neuron))\n\nSynonyms:\nemel/lazy/ml/neural_network.iterate\nemel/lazy/ml/neural_network iterate","url":"emel/lazy/ml/neural_network.html#iterate"},{"doc":"emel/lazy/ml/neural_network","title":"emel/lazy/ml/neural_network","content":"","url":"emel/lazy/ml/neural_network.html"},{"doc":"emel/lazy/ml/logistic_regression","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(#(ZList(Float), Bool)),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n  seed: Int,\n) -&gt; fn(ZList(Float)) -&gt; Bool\n\nSynonyms:\nemel/lazy/ml/logistic_regression.classifier\nemel/lazy/ml/logistic_regression classifier","url":"emel/lazy/ml/logistic_regression.html#classifier"},{"doc":"emel/lazy/ml/logistic_regression","title":"emel/lazy/ml/logistic_regression","content":"","url":"emel/lazy/ml/logistic_regression.html"},{"doc":"emel/lazy/ml/naive_bayes","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(Dict(String, String)),\n  discrete_attributes: ZList(String),\n  class: String,\n) -&gt; fn(Dict(String, String)) -&gt; String\n\nSynonyms:\nemel/lazy/ml/naive_bayes.classifier\nemel/lazy/ml/naive_bayes classifier","url":"emel/lazy/ml/naive_bayes.html#classifier"},{"doc":"emel/lazy/ml/naive_bayes","title":"emel/lazy/ml/naive_bayes","content":"","url":"emel/lazy/ml/naive_bayes.html"},{"doc":"emel/lazy/ml/perceptron","title":"classifier","content":"pub fn classifier(\n  dataset: ZList(#(ZList(Float), Bool)),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n) -&gt; fn(ZList(Float)) -&gt; Bool\n\nSynonyms:\nemel/lazy/ml/perceptron.classifier\nemel/lazy/ml/perceptron classifier","url":"emel/lazy/ml/perceptron.html#classifier"},{"doc":"emel/lazy/ml/perceptron","title":"emel/lazy/ml/perceptron","content":"","url":"emel/lazy/ml/perceptron.html"},{"doc":"emel/math/algebra","title":"cramer_solution","content":"pub fn cramer_solution(\n  coefficients: List(List(Float)),\n  constants: List(Float),\n) -&gt; Result(List(Float), String)\n The solution of a system of _linear equations_ by using Cramer&#39;s formula.\n\n erlang\n Coefficients = [\n   [1.0, 3.0, -2.0],\n   [3.0, 5.0, 6.0],\n   [2.0, 4.0, 3.0]\n ],\n Constants = [5.0, 7.0, 8.0],\n emel@math@algebra:cramer_solution(Coefficients, Constants).\n % {ok, [-15.0, 8.0, 2.0]}\n \nSynonyms:\nemel/math/algebra.cramer_solution\nemel/math/algebra cramer_solution","url":"emel/math/algebra.html#cramer_solution"},{"doc":"emel/math/algebra","title":"determinant","content":"pub fn determinant(matrix: List(List(Float))) -&gt; Float\n A value that can be computed from the elements of a square `matrix`.\n Geometrically, it can be viewed as the _scaling factor_ of the _linear transformation_ described by the `matrix`.\n\n erlang\n Matrix = [\n   [3.0, 8.0],\n   [4.0, 6.0]\n ],\n emel@math@algebra:determinant(Matrix).\n % -14.0\n \nSynonyms:\nemel/math/algebra.determinant\nemel/math/algebra determinant","url":"emel/math/algebra.html#determinant"},{"doc":"emel/math/algebra","title":"first_minor","content":"pub fn first_minor(\n  matrix: List(List(Float)),\n  i: Int,\n  j: Int,\n) -&gt; List(List(Float))\n The `first_minor` of a `matrix` obtained by removing just the `i`-row and the `j`-column from the `matrix`.\n It is required for calculating _cofactors_, which in turn are useful for computing both the _determinant_\n and the _inverse_ of square matrices. `i` and `j` are zero based.\n\n erlang\n Matrix = [\n   [5.0, -7.0, 2.0, 2.0],\n   [0.0, 3.0, 0.0, -4.0],\n   [-5.0, -8.0, 0.0, 3.0],\n   [0.0, 5.0, 0.0, -6.0],\n ],\n I = 0,\n J = 2,\n emel@math@algebra:first_minor(Matrix, I, J).\n % [\n %   [0.0, 3.0, -4.0],\n %   [-5.0, -8.0, 3.0],\n %   [0.0, 5.0, -6.0]\n % ]\n \nSynonyms:\nemel/math/algebra.first_minor\nemel/math/algebra first_minor","url":"emel/math/algebra.html#first_minor"},{"doc":"emel/math/algebra","title":"transpose","content":"pub fn transpose(matrix: List(List(a))) -&gt; List(List(a))\n A matrix whose rows are the columns of the original.\n\n erlang\n Matrix = [\n   [6.0, 1.0, 1.0],\n   [4.0, -2.0, 5.0],\n   [2.0, 8.0, 7.0]\n ],\n emel@math@algebra:transpose(Matrix).\n % [\n %   [6.0, 4.0, 2.0],\n %   [1.0, -2.0, 8.0],\n %   [1.0, 5.0, 7.0]\n % ]\n \nSynonyms:\nemel/math/algebra.transpose\nemel/math/algebra transpose","url":"emel/math/algebra.html#transpose"},{"doc":"emel/math/algebra","title":"emel/math/algebra","content":"","url":"emel/math/algebra.html"},{"doc":"emel/math/geometry","title":"centroid","content":"pub fn centroid(points: List(List(Float))) -&gt; List(Float)\n The _arithmetic mean_ position of the `points`.\n\n erlang\n Points = [\n   [0.0, 0.0],\n   [0.0, 1.0],\n   [1.0, 0.0],\n   [1.0, 1.0]\n ],\n emel@math@geometry:centroid(Points).\n % [0.5, 0.5]\n \nSynonyms:\nemel/math/geometry.centroid\nemel/math/geometry centroid","url":"emel/math/geometry.html#centroid"},{"doc":"emel/math/geometry","title":"dot_product","content":"pub fn dot_product(x: List(Float), y: List(Float)) -&gt; Float\n The sum of the products of the corresponding entries of `x` and `y`.\n\n erlang\n X = [1.0, 2.0, 3.0],\n Y = [4.0, -5.0, 6.0],\n emel@math@geometry:dot_product(X, Y).\n % 12.0\n \nSynonyms:\nemel/math/geometry.dot_product\nemel/math/geometry dot_product","url":"emel/math/geometry.html#dot_product"},{"doc":"emel/math/geometry","title":"euclidean_distance","content":"pub fn euclidean_distance(\n  x: List(Float),\n  y: List(Float),\n) -&gt; Float\n The ordinary straight-line distance between two points in _Euclidean space_.\n\n erlang\n X = [2.0, -1.0],\n Y = [-2.0, 2.0],\n emel@math@geometry:euclidean_distance(X, Y).\n % 5.0\n \nSynonyms:\nemel/math/geometry.euclidean_distance\nemel/math/geometry euclidean_distance","url":"emel/math/geometry.html#euclidean_distance"},{"doc":"emel/math/geometry","title":"magnitude","content":"pub fn magnitude(vector: List(Float)) -&gt; Float\n The _euclidean distance_ between the initial and the terminal point of the `vector`.\n\n erlang\n Vector = [6.0, 8.0],\n emel@math@geometry:magnitude(Vector).\n % 10.0\n \nSynonyms:\nemel/math/geometry.magnitude\nemel/math/geometry magnitude","url":"emel/math/geometry.html#magnitude"},{"doc":"emel/math/geometry","title":"nearest_neighbor","content":"pub fn nearest_neighbor(\n  point: List(Float),\n  neighbors: List(List(Float)),\n) -&gt; List(Float)\n The neighbor that is closest to the given `point`.\n\n erlang\n Point = [0.9, 0.0],\n Neighbors = [\n   [0.0, 0.0],\n   [0.0, 0.1],\n   [1.0, 0.0],\n   [1.0, 1.0]\n ],\n emel@math@geometry:nearest_neighbor(Point, Neighbors).\n % [1.0, 0.0]\n \nSynonyms:\nemel/math/geometry.nearest_neighbor\nemel/math/geometry nearest_neighbor","url":"emel/math/geometry.html#nearest_neighbor"},{"doc":"emel/math/geometry","title":"emel/math/geometry","content":"","url":"emel/math/geometry.html"},{"doc":"emel/math/statistics","title":"entropy","content":"pub fn entropy(probability_values: List(Float)) -&gt; Float\n A number that gives an idea of how random an outcome will be based on the `probability_values`\n of each of the possible outcomes in a situation.\n\n erlang\n ProbabilityValues = [0.8, 0.05, 0.05, 0.1],\n emel@math@statistics:entropy(ProbabilityValues).\n % 0.5109640474436812\n \nSynonyms:\nemel/math/statistics.entropy\nemel/math/statistics entropy","url":"emel/math/statistics.html#entropy"},{"doc":"emel/math/statistics","title":"mean_absolute_error","content":"pub fn mean_absolute_error(\n  predictions: List(Float),\n  observations: List(Float),\n) -&gt; Float\n A measure of difference between two continuous variables.\n\n erlang\n Predictions = [5.0, 1.0, 0.0, 0.5],\n Observations = [0.0, 1.0, -3.0, 0.5],\n emel@math@statistics:mean_absolute_error(Predictions, Observations).\n % 2.0\n \nSynonyms:\nemel/math/statistics.mean_absolute_error\nemel/math/statistics mean_absolute_error","url":"emel/math/statistics.html#mean_absolute_error"},{"doc":"emel/math/statistics","title":"emel/math/statistics","content":"","url":"emel/math/statistics.html"},{"doc":"emel/ml/decision_tree","title":"classifier","content":"pub fn classifier(\n  data: List(Dict(String, String)),\n  features: List(String),\n  class: String,\n) -&gt; fn(Dict(String, String)) -&gt; String\n Returns the function that classifies a point by using the _ID3 Algorithm_.\n\n erlang\n Data = [\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;low&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;bad&quot;, &quot;risk&quot; =&gt; &quot;high&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;moderate&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;unknown&quot;, &quot;risk&quot; =&gt; &quot;high&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;moderate&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;unknown&quot;, &quot;risk&quot; =&gt; &quot;moderate&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;low&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;unknown&quot;, &quot;risk&quot; =&gt; &quot;high&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;unknown&quot;, &quot;risk&quot; =&gt; &quot;low&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;adequate&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;unknown&quot;, &quot;risk&quot; =&gt; &quot;low&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;low&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;bad&quot;, &quot;risk&quot; =&gt; &quot;high&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;adequate&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;bad&quot;, &quot;risk&quot; =&gt; &quot;moderate&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;, &quot;risk&quot; =&gt; &quot;low&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;adequate&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;, &quot;risk&quot; =&gt; &quot;low&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;low&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;, &quot;risk&quot; =&gt; &quot;high&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;moderate&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;, &quot;risk&quot; =&gt; &quot;moderate&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;, &quot;risk&quot; =&gt; &quot;low&quot;},\n   #{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;moderate&quot;, &quot;debt&quot; =&gt; &quot;high&quot;, &quot;credit_history&quot; =&gt; &quot;bad&quot;, &quot;risk&quot; =&gt; &quot;high&quot;}\n ],\n Features = [&quot;collateral&quot;, &quot;income&quot;, &quot;debt&quot;, &quot;credit_history&quot;],\n Class = &quot;risk&quot;,\n F = emel@ml@decision_tree:classifier(Data, Features, Class),\n F(#{&quot;collateral&quot; =&gt; &quot;none&quot;, &quot;income&quot; =&gt; &quot;high&quot;, &quot;debt&quot; =&gt; &quot;low&quot;, &quot;credit_history&quot; =&gt; &quot;good&quot;}).\n % &quot;low&quot;\n \nSynonyms:\nemel/ml/decision_tree.classifier\nemel/ml/decision_tree classifier","url":"emel/ml/decision_tree.html#classifier"},{"doc":"emel/ml/decision_tree","title":"decision_tree","content":"pub fn decision_tree(\n  data: List(Dict(String, String)),\n  features: List(String),\n  class: String,\n) -&gt; List(Dict(String, String))\n The expanded _decision tree_ as a list of maps.\n It gets created with the _ID3 Algorithm_\n for the provided `data` and the specified `features`.\n\n erlang\n Data = [\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;}\n ],\n Features = [&quot;outlook&quot;, &quot;temperature&quot;, &quot;humidity&quot;, &quot;wind&quot;],\n Class = &quot;decision&quot;,\n emel@ml@decision_tree:decision_tree(Data, Features, Class).\n % [\n %   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n %   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n %   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n %   #{&quot;humidity&quot; =&gt; &quot;High&quot;, &quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n %   #{&quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;}\n % ]\n \nSynonyms:\nemel/ml/decision_tree.decision_tree\nemel/ml/decision_tree decision_tree","url":"emel/ml/decision_tree.html#decision_tree"},{"doc":"emel/ml/decision_tree","title":"emel/ml/decision_tree","content":" Uses a _decision tree_ to go from _observations_ about a point (represented in the branches)\n to conclusions about the point&#39;s discrete target value (represented in the leaves).","url":"emel/ml/decision_tree.html"},{"doc":"emel/utils/minigen","title":"seed","content":"pub fn seed() -&gt; Int\n\nSynonyms:\nemel/utils/minigen.seed\nemel/utils/minigen seed","url":"emel/utils/minigen.html#seed"},{"doc":"emel/utils/minigen","title":"emel/utils/minigen","content":"","url":"emel/utils/minigen.html"},{"doc":"emel/ml/k_means","title":"classifier","content":"pub fn classifier(\n  data: List(List(Float)),\n  k: Int,\n) -&gt; fn(List(Float)) -&gt; Int\n Returns the function that classifies a point by identifying the index of the _cluster_ it belongs to.\n\n erlang\n Points = [\n   [0.0, 0.0],\n   [4.0, 4.0],\n   [9.0, 9.0],\n   [4.3, 4.3],\n   [9.9, 9.9],\n   [4.4, 4.4],\n   [0.1, 0.1],\n ],\n K = 3,\n F = emel@ml@k_means:clusters(Points, K),\n F([4.7, 4.7]).\n % 1\n \nSynonyms:\nemel/ml/k_means.classifier\nemel/ml/k_means classifier","url":"emel/ml/k_means.html#classifier"},{"doc":"emel/ml/k_means","title":"clusters","content":"pub fn clusters(\n  points: List(List(Float)),\n  k: Int,\n) -&gt; List(List(List(Float)))\n The `points` get partitioned into `k` _clusters_ in which each _point_ belongs to the _cluster_ with the nearest _mean_.\n\n erlang\n Points = [\n   [0.0, 0.0],\n   [4.0, 4.0],\n   [9.0, 9.0],\n   [4.3, 4.3],\n   [9.9, 9.9],\n   [4.4, 4.4],\n   [0.1, 0.1],\n ],\n K = 3,\n emel@ml@k_means:clusters(Points, K).\n % [\n %   [[0.1, 0.1], [0.0, 0.0]],\n %   [[4.0, 4.0], [4.3, 4.3], [4.4, 4.4]],\n %   [[9.9, 9.9], [9.0, 9.0]],\n % ]\n \nSynonyms:\nemel/ml/k_means.clusters\nemel/ml/k_means clusters","url":"emel/ml/k_means.html#clusters"},{"doc":"emel/ml/k_means","title":"emel/ml/k_means","content":" Aims to partition n _observations_ into k _clusters_\n in which each _observation_ belongs to the _cluster_ with the nearest _mean_.","url":"emel/ml/k_means.html"},{"doc":"emel/ml/k_nearest_neighbors","title":"classifier","content":"pub fn classifier(\n  data: List(#(List(Float), String)),\n  k: Int,\n) -&gt; fn(List(Float)) -&gt; String\n Returns the function that classifies a point by finding the `k` nearest neighbors.\n\n erlang\n Data = [\n   {[7.0, 7.0], &quot;bad&quot;},\n   {[7.0, 4.0], &quot;bad&quot;},\n   {[3.0, 4.0], &quot;good&quot;},\n   {[1.0, 4.0], &quot;good&quot;}\n ],\n K = 3,\n F = emel@ml@k_nearest_neighbors:classifier(Data, K),\n F([3.0, 7.0]).\n % &quot;good&quot;\n \nSynonyms:\nemel/ml/k_nearest_neighbors.classifier\nemel/ml/k_nearest_neighbors classifier","url":"emel/ml/k_nearest_neighbors.html#classifier"},{"doc":"emel/ml/k_nearest_neighbors","title":"k_nearest_neighbors","content":"pub fn k_nearest_neighbors(\n  data: List(#(List(Float), a)),\n  point: List(Float),\n  k: Int,\n) -&gt; List(#(List(Float), a))\n It searches through the `data` and returns the `k` most similar items to the `point`.\n\n erlang\n Data = [\n   {[7.0, 7.0], &quot;bad&quot;},\n   {[7.0, 4.0], &quot;bad&quot;},\n   {[3.0, 4.0], &quot;good&quot;},\n   {[1.0, 4.0], &quot;good&quot;}\n ],\n Point = [3.0, 7.0],\n K = 3,\n emel@ml@k_nearest_neighbors:k_nearest_neighbors(Data, Point, K).\n % [\n %   {[3.0, 4.0], &quot;good&quot;},\n %   {[1.0, 4.0], &quot;good&quot;},\n %   {[7.0, 7.0], &quot;bad&quot;}\n % ]\n \nSynonyms:\nemel/ml/k_nearest_neighbors.k_nearest_neighbors\nemel/ml/k_nearest_neighbors k_nearest_neighbors","url":"emel/ml/k_nearest_neighbors.html#k_nearest_neighbors"},{"doc":"emel/ml/k_nearest_neighbors","title":"predictor","content":"pub fn predictor(\n  data: List(#(List(Float), Float)),\n  k: Int,\n) -&gt; fn(List(Float)) -&gt; Float\n Returns the function that calculates the average value\n of the `dependent variable` for the `k` nearest neighbors.\n\n erlang\n Data = [\n   {[0.0, 0.0, 0.0], 0.0},\n   {[0.5, 0.5, 0.5], 1.5},\n   {[1.0, 1.0, 1.0], 3.0},\n   {[1.5, 1.5, 1.5], 4.5},\n   {[2.0, 2.0, 2.0], 6.0},\n   {[2.5, 2.5, 2.5], 7.5},\n   {[3.0, 3.3, 3.0], 9.0}\n ],\n K = 2,\n F = emel@ml@k_nearest_neighbors:predictor(Data, K),\n F([1.725, 1.725, 1.725]).\n % 5.25\n \nSynonyms:\nemel/ml/k_nearest_neighbors.predictor\nemel/ml/k_nearest_neighbors predictor","url":"emel/ml/k_nearest_neighbors.html#predictor"},{"doc":"emel/ml/k_nearest_neighbors","title":"emel/ml/k_nearest_neighbors","content":" A non-parametric method used for _classification_ and _regression_.\n In both cases, the input consists of the k closest training examples in the feature space.","url":"emel/ml/k_nearest_neighbors.html"},{"doc":"emel/ml/linear_regression","title":"predictor","content":"pub fn predictor(\n  data: List(#(List(Float), Float)),\n) -&gt; fn(List(Float)) -&gt; Float\n Returns the linear function that predicts the value of the _dependent variable_.\n\n erlang\n Data = [\n   {[1.794638, 15.15426], 0.510998918},\n   {[3.220726, 229.6516], 105.6583692},\n   {[5.780040, 3480.201], 1776.99}\n ],\n F = emel@ml@linear_regression:predictor(Data),\n F([3.0, 230.0]).\n % 106.74114058686602\n \nSynonyms:\nemel/ml/linear_regression.predictor\nemel/ml/linear_regression predictor","url":"emel/ml/linear_regression.html#predictor"},{"doc":"emel/ml/linear_regression","title":"regression_coefficients","content":"pub fn regression_coefficients(\n  points: List(List(Float)),\n) -&gt; Result(List(Float), String)\n The list of the _predictor function_&#39;s _coefficients_ based on _observations_ (`points`).\n\n erlang\n Points = [[1.0, 1.0, 1.0], [1.0, 2.0, 3.0], [2.0, 1.0, 0.0]],\n emel@ml@linear_regression:regression_coefficients(Points).\n % {ok, [0.0, -1.0, 2.0]}\n \nSynonyms:\nemel/ml/linear_regression.regression_coefficients\nemel/ml/linear_regression regression_coefficients","url":"emel/ml/linear_regression.html#regression_coefficients"},{"doc":"emel/ml/linear_regression","title":"emel/ml/linear_regression","content":" A linear approach to modelling the relationship between\n a list of _independent variables_ and a _dependent variable_.","url":"emel/ml/linear_regression.html"},{"doc":"emel/ml/logistic_regression","title":"classifier","content":"pub fn classifier(\n  data: List(#(List(Float), Bool)),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n) -&gt; fn(List(Float)) -&gt; Bool\n Returns the function that classifies a poin by using the _Logistic Regression Algorithm_.\n\n erlang\n Data = [\n   {[0.0, 0.0], false},\n   {[0.0, 1.0], false},\n   {[1.0, 0.0], false},\n   {[1.0, 1.0], true}\n ],\n LearningRate = 0.01,\n ErrorThreshold = 0.1,\n MaxIterations = 1000,\n F = emel@ml@logistic_regression:classifier(Data, LearningRate, ErrorThreshold, MaxIterations),\n F([1.0, 1.0]).\n % true\n \nSynonyms:\nemel/ml/logistic_regression.classifier\nemel/ml/logistic_regression classifier","url":"emel/ml/logistic_regression.html#classifier"},{"doc":"emel/ml/logistic_regression","title":"emel/ml/logistic_regression","content":" A classification algorithm used to assign observations to a set of two classes.\n It transforms its output by using the _logistic sigmoid function_ to return a probability value\n which can then be mapped to the classes.","url":"emel/ml/logistic_regression.html"},{"doc":"emel/ml/naive_bayes","title":"classifier","content":"pub fn classifier(\n  data: List(Dict(String, String)),\n  features: List(String),\n  class: String,\n) -&gt; fn(Dict(String, String)) -&gt; String\n Returns the function that classifies an item by using the _Naive Bayes Algorithm_.\n\n erlang\n Data = [\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;No&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Cool&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Overcast&quot;, &quot;temperature&quot; =&gt; &quot;Hot&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Weak&quot;, &quot;decision&quot; =&gt; &quot;Yes&quot;},\n   #{&quot;outlook&quot; =&gt; &quot;Rain&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;High&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;, &quot;decision&quot; =&gt; &quot;No&quot;}\n ],\n Features = [&quot;outlook&quot;, &quot;temperature&quot;, &quot;humidity&quot;, &quot;wind&quot;],\n Class = &quot;decision&quot;,\n F = emel@ml@naive_bayes:classifier(Data, Features, Class),\n F(#{&quot;outlook&quot; =&gt; &quot;Sunny&quot;, &quot;temperature&quot; =&gt; &quot;Mild&quot;, &quot;humidity&quot; =&gt; &quot;Normal&quot;, &quot;wind&quot; =&gt; &quot;Strong&quot;}).\n % &quot;Yes&quot;\n \nSynonyms:\nemel/ml/naive_bayes.classifier\nemel/ml/naive_bayes classifier","url":"emel/ml/naive_bayes.html#classifier"},{"doc":"emel/ml/naive_bayes","title":"emel/ml/naive_bayes","content":" A simple _probabilistic classifier_ based on applying _Bayes&#39; theorem_\n with naive independence assumptions between the features.\n It makes classifications using the maximum _posteriori_ decision rule in a Bayesian setting.","url":"emel/ml/naive_bayes.html"},{"doc":"emel/ml/neural_network","title":"classifier","content":"pub fn classifier(\n  data: List(#(List(Float), String)),\n  hidden_layers: List(Int),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n) -&gt; fn(List(Float)) -&gt; String\n Returns the function that classifies a point by using the _Neural Network Framework_.\n\n erlang\n Data = [\n   {[0.8, 0.0, 0.0], &quot;x&quot;},\n   {[0.0, 0.9, 0.0], &quot;y&quot;},\n   {[0.0, 0.0, 0.8], &quot;z&quot;}\n ],\n HiddenLayers = [4],\n LearningRate = 0.01,\n ErrorThreshold = 0.1,\n MaxIterations = 1000,\n F = emel@ml@neural_network:classifier(Data, HiddenLayers, LearningRate, ErrorThreshold, MaxIterations),\n F([0.0, 0.8, 0.0]).\n % &quot;y&quot;\n \nSynonyms:\nemel/ml/neural_network.classifier\nemel/ml/neural_network classifier","url":"emel/ml/neural_network.html#classifier"},{"doc":"emel/ml/neural_network","title":"emel/ml/neural_network","content":" A collection of connected neurons, that looks like a biological brain.\n Each connection, can transmit a signal from one neuron to another.","url":"emel/ml/neural_network.html"},{"doc":"emel/ml/perceptron","title":"classifier","content":"pub fn classifier(\n  data: List(#(List(Float), Bool)),\n  learning_rate: Float,\n  error_threshold: Float,\n  max_iterations: Int,\n) -&gt; fn(List(Float)) -&gt; Bool\n Returns the function that classifies a point by using the _Perceptron Algorithm_.\n\n erlang\n Data = [\n   {[0.0, 0.0], false},\n   {[0.0, 1.0], true},\n   {[1.0, 0.0], true},\n   {[1.0, 1.0], true}\n ],\n LearningRate = 0.01,\n ErrorThreshold = 0.1,\n MaxIterations = 1000,\n F = emel@ml@perceptron:classifier(Data, LearningRate, ErrorThreshold, MaxIterations),\n F([0.0, 0.0]).\n % false\n \nSynonyms:\nemel/ml/perceptron.classifier\nemel/ml/perceptron classifier","url":"emel/ml/perceptron.html#classifier"},{"doc":"emel/ml/perceptron","title":"emel/ml/perceptron","content":" A _binary classification algorithm_ that makes its predictions based on a _linear predictor function_\n combining a set of weights with the features.","url":"emel/ml/perceptron.html"}]);